import 'dart:io';

import 'package:bosun/bosun.dart';

import '../core/structure.dart';
import '../functions/binding/bindings.dart';
import '../functions/binding/controller.dart';
import '../functions/binding/find_file_by_type.dart';
import '../functions/create/create_file.dart';
import '../samples/impl/provider.dart';
import '../utils/logger/log_utils.dart';

class ProviderCommand extends Command {
  ProviderCommand()
      : super(
          command: 'provider',
          description: 'Create a new provider for Megaleios projects. 🔨',
          example: 'me provider login',
        );

  @override
  void run(List<String> args, Map<String, dynamic> flags) {
    if (args.isEmpty) {
      LogService.error("You must provide a provider name.");
      return;
    }
    _makeNewProvider(featureName: args.first);
  }

  void _makeNewProvider({required String featureName}) {
    final featurePath = 'lib/app/data';
    Directory(featurePath).createSync(recursive: true);
    _writeFiles(featurePath, featureName, overwrite: false);
  }

  void _writeFiles(
    String path,
    String featureName, {
    required bool overwrite,
  }) {
    final providerFile = createFile(
      path: path,
      name: featureName,
      command: 'provider',
      folderName: 'providers',
      sample: ProviderSample(
        '',
        featureName,
        overwrite: overwrite,
      ),
    );
    final providerPath = Structure.pathToDirImport(providerFile.path);
    var binindingPath = findFileFromNameAndType(
      'lib/app/modules/$featureName/bindings/',
      featureName,
      typeFile: TypeFile.binding,
    );
    if (binindingPath == null) {
      LogService.error('No binding found for $featureName');
      return;
    }

    if (binindingPath.isNotEmpty) {
      addDependencyToBinding(binindingPath, featureName);
    }

    final controllerPath = findFileFromNameAndType(
      'lib/app/modules/$featureName/controllers/',
      featureName,
      typeFile: TypeFile.controller,
    );

    if (controllerPath == null) {
      LogService.error('No controller found for $featureName');
      return;
    }

    if (controllerPath.isNotEmpty) {
      addConstructorToController(
        path: controllerPath,
        providerName: featureName,
      );
    }

    LogService.success("Created provider $providerPath");
  }
}
